@@ -39,7 +39,6 @@
 
 #define IBUS_COMPOSE_TABLE_MAGIC "IBusComposeTable"
 #define IBUS_COMPOSE_TABLE_VERSION (4)
-#define X11_DATADIR X11_DATA_PREFIX "/share/X11/locale"
 #define IBUS_MAX_COMPOSE_ALGORITHM_LEN 9
 
 typedef struct {
@@ -285,7 +284,7 @@ expand_include_path (const char *include_path) {
             case 'S': /* system compose dir */
                 o = out;
                 former = g_strndup (head, i - head);
-                out = g_strdup_printf ("%s%s%s", o, former, X11_DATADIR);
+                out = g_strdup_printf ("%s%s%s", o, former, X11_LOCALEDATADIR);
                 head = i + 2;
                 g_free (o);
                 g_free (former);
@@ -397,7 +396,7 @@ get_en_compose_file (void)
     char * const *sys_lang = NULL;
     char *path = NULL;
     for (sys_lang = sys_langs; *sys_lang; sys_lang++) {
-        path = g_build_filename (X11_DATADIR, *sys_lang, "Compose", NULL);
+        path = g_build_filename (X11_LOCALEDATADIR, *sys_lang, "Compose", NULL);
         if (g_file_test (path, G_FILE_TEST_EXISTS))
             break;
         g_clear_pointer (&path, g_free);
@@ -479,7 +478,8 @@ ibus_compose_list_parse_file (const char *compose_file,
             }
         }
         if (include && *include) {
-            GList *rest = ibus_compose_list_parse_file (include,
+            GList *rest = ibus_compose_list_parse_file (
+                    include,
                     max_compose_len);
             if (rest) {
                 compose_list = g_list_concat (compose_list, rest);
@@ -499,12 +499,12 @@ ibus_compose_list_check_duplicated (GList  *compose_list,
                                     GSList *compose_tables)
 {
     GList *list;
-    static guint16 *keysyms;
+    static guint *keysyms;
     GList *removed_list = NULL;
     IBusComposeData *compose_data;
     GString *output = g_string_new ("");
 
-    keysyms = g_new (guint16, max_compose_len + 1);
+    keysyms = g_new (guint, max_compose_len + 1);
 
     for (list = compose_list; list != NULL; list = list->next) {
         int i;
@@ -524,7 +524,7 @@ ibus_compose_list_check_duplicated (GList  *compose_list,
 
         for (i = 0; i < max_compose_len + 1; i++) {
             gunichar codepoint = compose_data->sequence[i];
-            keysyms[i] = (guint16) codepoint;
+            keysyms[i] = (guint)codepoint;
 
             if (codepoint == 0)
                 break;
@@ -601,6 +601,34 @@ ibus_compose_list_check_duplicated (GList  *compose_list,
 }
 
 
+/*
+ * Actual typed keysyms have a flag against the definition.
+ * https://gitlab.freedesktop.org/xorg/lib/libx11/-/blob/master/nls/en_US.UTF-8/Compose.pre#L4559
+ */
+guint
+ibus_compose_key_flag (guint key)
+{
+    const char *name;
+    if (key <= 0xff)
+        return 0;
+    name = ibus_keyval_name (key);
+    /* If name is null, the key sequence is expressed as "<Uxxxx>" format in
+     * the Compose file and the typed keysym has the flag.
+     */
+    if (!name)
+        return 0x1000000;
+    /* "<Pointer_EnableKeys>" is not described in the Compose file but <UFEF9>
+     * in the file.
+     */
+    if (*name == 'P' && *(name + 1) == 'o' && *(name + 2) == 'i')
+        return 0x1000000;
+    /* Other names, "<Multi_key>", "<dead_*>", "<Cyrillic_*>" are described in
+     * the Compose file.
+     */
+    return 0;
+}
+
+
 static int
 ibus_compose_data_compare (gpointer a,
                            gpointer b,
@@ -620,6 +648,8 @@ ibus_compose_data_compare (gpointer a,
         gunichar code_a = compose_data_a->sequence[i];
         gunichar code_b = compose_data_b->sequence[i];
 
+        code_a += ibus_compose_key_flag (code_a);
+        code_b += ibus_compose_key_flag (code_b);
         if (code_a != code_b)
             return code_a - code_b;
         if (code_a == 0 && code_b == 0)
@@ -733,8 +763,8 @@ compose_data_to_variant (gconstpointer compose_data,
                          gboolean reverse_endianness,
                          GError **error)
 {
-    guint16 *compose_data16 = NULL;
-    guint32 *compose_data32 = NULL;
+    guint16 data16;
+    guint32 data32;
     guint16 *target_data16 = NULL;
     guint32 *target_data32 = NULL;
     gsize i, length;
@@ -760,18 +790,20 @@ compose_data_to_variant (gconstpointer compose_data,
                              "Failed to malloc");
                 return NULL;
             }
-            compose_data32 = (guint32*)compose_data;
-            for (i = 0; i < length; i++)
-                target_data32[i] = GUINT32_SWAP_LE_BE (compose_data32[i]);
+            for (i = 0; i < length; i++) {
+                memcpy(&data32, (char *)compose_data + i * sizeof (data32), sizeof (data32));
+                target_data32[i] = GUINT32_SWAP_LE_BE (data32);
+            }
         } else {
             if (!(target_data16 = g_new0 (guint16, length))) {
                 g_set_error (error, IBUS_ERROR, IBUS_ERROR_FAILED,
                              "Failed to malloc");
                 return NULL;
             }
-            compose_data16 = (guint16*)compose_data;
-            for (i = 0; i < length; i++)
-                target_data16[i] = GUINT16_SWAP_LE_BE (compose_data16[i]);
+            for (i = 0; i < length; i++) {
+                memcpy(&data16, (char *)compose_data + i * sizeof (data16), sizeof (data16));
+                target_data16[i] = GUINT16_SWAP_LE_BE (data16);
+            }
         }
     } else {
         if (is_32bit)
@@ -1366,8 +1398,9 @@ ibus_compose_table_new_with_list (GList   *compose_list,
         if (is_32bit) {
             for (j = 0; compose_data->values[j]; j++) {
                 g_assert (v_index_32bit + j <  v_size_32bit);
-                ibus_compose_seqs_32bit_second[v_index_32bit + j] =
-                        compose_data->values[j];
+                memcpy(&ibus_compose_seqs_32bit_second[v_index_32bit + j],
+                       &compose_data->values[j],
+                       sizeof *ibus_compose_seqs_32bit_second);
             }
             g_assert (m + 1 < (s_size_total - s_size_16bit) * n_index_stride);
             ibus_compose_seqs_32bit_first[m++] = j;
@@ -1540,13 +1573,16 @@ static int
 compare_seq (const void *key, const void *value)
 {
     int i = 0;
-    const guint16 *keysyms = key;
+    const guint *keysyms = key;
     const guint16 *seq = value;
 
     while (keysyms[i]) {
-        if (keysyms[i] < seq[i])
+        guint typed_key = keysyms[i];
+        guint saved_key = (guint)seq[i];
+        guint flag = ibus_compose_key_flag (saved_key);
+        if (typed_key < (saved_key + flag))
             return -1;
-        else if (keysyms[i] > seq[i])
+        else if (typed_key > (saved_key + flag))
             return 1;
 
         i++;
@@ -1556,9 +1592,40 @@ compare_seq (const void *key, const void *value)
 }
 
 
+/**
+ * ibus_compose_table_check:
+ * @table: An #IBusComposeTableEx.
+ * @compose_buffer: (array length=n_compose):
+                    A candidate typed key sequence to generate compose chars.
+ * @n_compose: The length of compose_buffer.
+ * @compose_finish: If typed key sequence is finished for the compose chars.
+ * @compose_match: If typed key sequence is matched partically.
+ * @output: Matched compse chars.
+ * @is_32bit: The type of #IBusComposeTableEx.
+ *
+ * If the current @comopse_buffer matches a compose sequence partically in
+ * #IBusComposeTableEx, return %TRUE otherwise %FALSE.
+ * If the matched compose sequence can be converted to a Unicode string,
+ * %TRUE is set to @compose_match and the converted string is assgined to
+ * @output to update the preedit text.
+ * If the current @comopse_buffer matches a compose sequence fully in
+ * #IBusComposeTableEx, %TRUE is set to @compose_finish and the generated
+ * compose character is assigned to @output.
+ *
+ * #IBusComposeTableEx includes 16bit key sequences and the corresponded
+ * compose characters. #IBusComposeTableEx has two types, one is
+ * 16bit and one compose char and another is 32bit or more than two chars.
+ * If #IBusComposeTableEx is 16bit and one compose char, @is_32bit is %TRUE.
+ *
+ * @compose_buffer is the typed key sequence but not limitted to 16bit.
+ * E.g.
+ * Shift + t is 0x100fef9 keysym with Arabic keyboard.
+ * Shift + NumLock is 0xfef9 keysym with keypad:pointerkeys XKB option.
+ * So compare_seq() compares the 32bit key sequence and 16bit one.
+ */
 gboolean
 ibus_compose_table_check (const IBusComposeTableEx *table,
-                          guint16                  *compose_buffer,
+                          guint                    *compose_buffer,
                           int                       n_compose,
                           gboolean                 *compose_finish,
                           gboolean                 *compose_match,
@@ -1747,7 +1814,7 @@ check_normalize_nfc (gunichar* combination_buffer, int n_compose)
 
 
 gboolean
-ibus_check_algorithmically (const guint16 *compose_buffer,
+ibus_check_algorithmically (const guint   *compose_buffer,
                             int            n_compose,
                             gunichar      *output_char)
 
@@ -1816,7 +1883,7 @@ ibus_check_algorithmically (const guint16 *compose_buffer,
 
 
 gunichar
-ibus_keysym_to_unicode (guint16   keysym,
+ibus_keysym_to_unicode (guint     keysym,
                         gboolean  combining,
                         gboolean *need_space) {
 #define CASE(keysym_suffix, unicode, sp)                                      \
