@@ -308,12 +308,15 @@ ibus_init (void)
     IBUS_TYPE_TEXT;
     IBUS_TYPE_ATTRIBUTE;
     IBUS_TYPE_ATTR_LIST;
-    IBUS_TYPE_LOOKUP_TABLE;
     IBUS_TYPE_COMPONENT;
+    IBUS_TYPE_EMOJI_DATA;
     IBUS_TYPE_ENGINE_DESC;
+    IBUS_TYPE_LOOKUP_TABLE;
     IBUS_TYPE_OBSERVED_PATH;
     IBUS_TYPE_REGISTRY;
     IBUS_TYPE_X_EVENT;
+    IBUS_TYPE_UNICODE_BLOCK;
+    IBUS_TYPE_UNICODE_DATA;
     _ibus_register_resource ();
 }
 
