@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2018-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2018-2019 Red Hat, Inc.
+ * Copyright (C) 2018-2021 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2018-2021 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -472,7 +472,12 @@ ibus_unicode_data_save (const gchar *path,
 
     dir = g_path_get_dirname (path);
     if (g_strcmp0 (dir, ".") != 0 && g_stat (dir, &buf) != 0) {
-        g_mkdir_with_parents (dir, 0777);
+        errno = 0;
+        if (g_mkdir_with_parents (dir, 0777)) {
+            g_warning ("Failed to mkdir %s: %s", dir, g_strerror (errno));
+            return;
+        }
+
     }
     g_free (dir);
     if (!g_file_set_contents (path, contents, length, &error)) {
@@ -967,7 +972,11 @@ ibus_unicode_block_save (const gchar *path,
 
     dir = g_path_get_dirname (path);
     if (g_strcmp0 (dir, ".") != 0 && g_stat (dir, &buf) != 0) {
-        g_mkdir_with_parents (dir, 0777);
+        errno = 0;
+        if (g_mkdir_with_parents (dir, 0777)) {
+            g_warning ("Failed to mkdir %s: %s", dir, g_strerror (errno));
+            return;
+        }
     }
     g_free (dir);
     if (!g_file_set_contents (path, contents, length, &error)) {
